package com.syc.slm.tools.utils;

import com.syc.slm.tools.config.InfluxdbConfig;
import okhttp3.OkHttpClient;
import org.apache.http.ssl.SSLContexts;
import org.influxdb.InfluxDB;
import org.influxdb.InfluxDBFactory;
import org.influxdb.dto.BatchPoints;
import org.influxdb.dto.Point;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.net.ssl.*;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 创建时间：2018-01-28
 * <p>修改时间：2018-01-28
 * <p>类说明：TOTO
 *
 * @author luke
 * @version 1.0
 */
@Component
public class InfluxUtils {


    @Autowired
    private InfluxdbConfig influxdbConfig;

    private InfluxDB influxDB;

    @PostConstruct
    private void init() {
        if (influxdbConfig.port.equals("8635")) {
            // 华为云: https + getBuilder
            influxDB = InfluxDBFactory.connect(String.format("https://%s:%s", influxdbConfig.host, influxdbConfig.port),
                                               influxdbConfig.username,
                                               influxdbConfig.password,
                                               getBuilder());
        } else {
            influxDB = InfluxDBFactory.connect(String.format("http://%s:%s", influxdbConfig.host, influxdbConfig.port),
                                               influxdbConfig.username,
                                               influxdbConfig.password);
        }
    }

    /**
     * 插入
     *
     * @param measurement 表
     * @param tags        标签
     * @param fields      字段
     */
    public void insert(String database, String measurement, Map<String, String> tags, Map<String, Object> fields) {
        Point.Builder builder = Point.measurement(measurement);
        builder.tag(tags);
        builder.fields(fields);

        influxDB.write(database, "", builder.build());
    }

    /**
     * 批量插入
     *
     * @param measurement 表
     * @param tagsList    标签
     * @param fieldsList  字段
     */
    public void insert(String database,
        String measurement,
        List<Map<String, String>> tagsList,
        List<Map<String, Object>> fieldsList) {
        BatchPoints.Builder bpBuilder = BatchPoints.database(database);
        if (fieldsList != null && fieldsList.size() > 0) {
            for (int i = 0; i < fieldsList.size(); i++) {
                Point.Builder builder = Point.measurement(measurement);
                builder.tag(tagsList.get(i));
                builder.fields(fieldsList.get(i));
                bpBuilder.point(builder.build());
            }
        }

        influxDB.write(bpBuilder.build());
    }

    /**
     * 按时间插入数据
     *
     * @param measurement 表
     * @param tags        标签
     * @param fields      字段
     */
    public void insertByTime(String database,
        String measurement,
        Map<String, String> tags,
        Map<String, Object> fields,
        long time) {
        Point.Builder builder = Point.measurement(measurement);
        builder.tag(tags);
        builder.fields(fields);
        builder.time(time, TimeUnit.MILLISECONDS);

        influxDB.write(database, "", builder.build());
    }

    /**
     * 查询
     *
     * @param command 查询语句
     * @return
     */
    public QueryResult query(String database, String command) {
        return influxDB.query(new Query(command, database));
    }

    /**
     * 免除SSL
     *
     * @return
     */
    private static OkHttpClient.Builder getBuilder() {
        OkHttpClient.Builder builder = new OkHttpClient.Builder();
        builder.sslSocketFactory(defaultSslSocketFactory(), defaultTrustManager());
        builder.hostnameVerifier(noopHostnameVerifier());
        return builder;
    }

    private static X509TrustManager defaultTrustManager() {
        return new X509TrustManager() {
            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[0];
            }

            @Override
            public void checkClientTrusted(X509Certificate[] certs, String authType) {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] certs, String authType) {
            }
        };
    }

    private static SSLSocketFactory defaultSslSocketFactory() {
        try {
            SSLContext sslcontext = SSLContexts.createDefault();
            sslcontext.init(null, new TrustManager[] {defaultTrustManager()}, new SecureRandom());
            return sslcontext.getSocketFactory();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static HostnameVerifier noopHostnameVerifier() {
        return new HostnameVerifier() {
            @Override
            public boolean verify(final String s, final SSLSession sslSession) {
                return true;
            }
        };
    }
}
